
/* BEGIN COPYRIGHT NOTICE

Copyright (c) 1992 by John Sarraille and Peter DiFalco
(john@ishi.csustan.edu)

Copyright (c) 1998 by John Sarraille
(john@ishi.csustan.edu)

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that copyright notice and this permission
notice appear in supporting documentation.

The algorithm used in this program was inspired by the paper
entitled "A Fast Algorithm To Determine Fractal Dimensions By
Box Counting", which was written by Liebovitch and Toth, and
which appeared in the journal "Physics Letters A", volume 141,
pp 386-390, (1989).

This program is not warranted: use at your own risk.

END COPYRIGHT NOTICE */


/*
   FD3 PROGRAM FILE

Filename:  coversf.c

OUTLINE OF CALLS:

FindRefRange +
  GetRefSatPt +
  GetRefFMPt +
  ShowRefRange +
InitRefData +
GetRefStatRAM +
  gmalloc  
XferSweepRes +
DoRescales +
  ClearQ
  LoadRAM
  SortRAM
  StdDataSweep 
  XferSweepRes +
ReportRefCovD +
  CSTtitles 
  RCSTbody +
    MakeStarBar 
  RfTwoPtEstTitles +
  RfTwoPtEstBody +
    MakeStarBar 
*/

#include "fd.h"

/* 
    THE FUNCTIONS IN THIS FILE: 
*/
void  FindRefRange    () ;
void  GetRefSatPt     () ;
void  GetRefFMPt      () ;
void  ShowRefRange    () ;
void  InitRefData     () ;
void  GetRefStatRAM   () ;
void  XferSweepRes    () ;
void  DoRescales      () ;
void  ReportRefCovD   () ;
void  RCSTbody        () ;
void  RfTwoPtEstTitles() ;
void  RfTwoPtEstBody  () ;
/*   
      THE FUNCTIONS NOT IN THIS FILE,
      BUT CALLED FROM HERE:
*/
extern void   * gmalloc       () ;     /*  in mem.c      */
extern void     ClearQ        () ;     /*  in queue.c    */
extern void     LoadRAM       () ;     /*  in datarep.c  */
extern void     SortRAM       () ;     /*  in datarep.c  */
extern void     StdDataSweep  () ;     /*  in covers.c   */
extern void     CSTtitles     () ;     /*  in covers.c   */
extern void     MakeStarBar   () ;     /*  in covers.c   */

/* ######################################################### */
/*                        FindRefRange                       */
/* ######################################################### */
/*
    THIS IS A VERSION OF "FindRange" FOR USE WITH THE fine (-f)
    OPTION.  IT REALLY DOESN'T DIFFER FROM "FindRange"
    SUBSTANTIVELY.  PRIMARILY WE ARE USING THIS AS A
    CONVENIENCE UNTIL WE RE-ORGANIZE THE DATA STRUCTURES THAT
    CONTAIN COVERING DATA.
*/
void  FindRefRange  (rfcdP, cmdPtr)
        refinedCDtype * rfcdP   ;
       commandType    * cmdPtr  ;       
{
   if ( (cmdPtr->placeBars) )
   {
     if (     ((cmdPtr->saturationPoint) < (cmdPtr->fineMeshPoint))
           && ((cmdPtr->fineMeshPoint) < (rfcdP->numCellSizes) )
        )
     
     {
       (rfcdP->saturationPoint) = (cmdPtr->saturationPoint) ;
       (rfcdP->fineMeshPoint) = (cmdPtr->fineMeshPoint) ;
     }
     else
     {
        printf("\n") ;
        printf("PROBLEM in function FindRefRange: \n") ;
        printf("The cell-size bounds that you requested with\n") ;
        printf("the -p option are out of range.\n") ;
        printf("\n") ;
        fflush(stdout) ;
        exit(EXIT_FAILURE) ;
     }
   }
   else
   {
     GetRefSatPt (rfcdP) ;
     GetRefFMPt (rfcdP) ;
   }
   if (
        ((cmdPtr->debugging) || (cmdPtr->checking))
      )
      ShowRefRange (rfcdP) ;
   return ;
}
/* ######################################################### */
/*                       GetRefSatPt                         */
/* ######################################################### */
/*
   GET THE SATURATION POINT FOR THE REFINED ESTIMATE.
*/
void GetRefSatPt (rfcdP) 
        refinedCDtype * rfcdP ;
{
   int stop ;
   uLong i ;

   double log3 = log(3.0)/log(2.0), logCutFac ;

   if    ( (rfcdP->embedDim) > log3) 
         logCutFac = (rfcdP->embedDim) ;
   else  logCutFac = log3 ;

   stop = FALSE ;
   i = (rfcdP->numCellSizes) ;
   while (     !stop
           &&    (  -rfcdP->negLogCellCount[i] <
                 (-rfcdP->negLogCellCount[0] - logCutFac) ))
   {
      stop = (i == 0) ;
      if (i != 0) i-- ;
   }
   if    (i > 0)
         (rfcdP->saturationPoint) = i+1-(rfcdP->numSets);
   else  (rfcdP->saturationPoint) = 0 ;  

  return ;
}
/* ######################################################### */
/*                       GetRefFMPt                          */
/* ######################################################### */
/*
    GET THE FINE-MESH POINT FOR THE REFINED ESTIMATE.  HERE WE
    SCAN FROM LARGE CELL SIZES DOWN UNTIL FINDING A SIZE FOR
    WHICH THERE EXISTS TEN OR MORE CELLS, OR UNTIL WE GET TO A
    CELL SIZE THAT IS ONE FOURTH THE SIZE OF THE GREATEST
    CELL.  WE PREDICT THIS WILL ALWAYS OUTPUT 31 OR 30 AS THE
    FINE-MESH POINT!  BUT WE WILL WANT TO ADJUST THIS CODE.
*/
void GetRefFMPt (rfcdP)
      refinedCDtype * rfcdP ;
{
   int      stop = FALSE, ulbn = rfcdP->ulongBitNum ;
   uLong   i, ed = rfcdP->embedDim ;
   double  logTen = log(10.0)/log(2.0) ;

   i = (rfcdP->numCellSizes) - 1 ;
   while (     !stop
           &&  (ed * (ulbn - (rfcdP->logCellSize)[i]) < logTen )
           &&  (rfcdP->logCellSize[i] > ulbn-2) )
   {
      i-- ;
      stop = (i == 0) ;
   }
   (rfcdP->fineMeshPoint) = i ;
   return ;
}
/* ######################################################### */
/*                      ShowRefRange                         */
/* ######################################################### */
/*
     A PROCEDURE TO SHOW WHAT RANGE OF CELL INDEXES WAS CHOSEN
     FOR DOING THE LEAST-SQUARES ESTIMATES.  THIS IS A SPECIAL
     VERSION OF "ShowRange" FOR USE WITH THE -f OPTION.
*/
void ShowRefRange (rfcdP)
        refinedCDtype * rfcdP   ;

{
   printf("\n") ;
   printf("The saturationPoint is: ") ;
   printf("%lu\n", rfcdP->saturationPoint) ;
   printf("The fineMeshPoint is: ") ;
   printf("%lu\n", rfcdP->fineMeshPoint) ;
   return ;
}
/* ######################################################### */
/*                        InitRefData                        */
/* ######################################################### */
/*
    INITIALIZE A FEW FIELDS IN THE STRUCTURE USED TO KEEP TRACK
    OF THE REFINED COVERING STATISTICS.  WE RECORD refDepth,
    THE EXPONENT OF THE POWER OF 2 WHOSE RECIPROCAL IS USED TO
    INCREMENT THE SCALE MODIFICATIONS.  FOR EXAMPLE, WHEN
    refDepth IS 5, THE SCALES ARE MODIFIED IN STEPS OF 1/32.
    WE THEN DETERMINE numSets, WHICH IS THE NUMBER OF SCALE
    INCREMENTS TO BE DONE. Denom IS THE POWER 2^^refDepth.
    (rfcdP->numCellSizes) IS THE TOTAL NUMBER OF CELL SIZES
    THAT WILL BE GENERATED.
*/
void  InitRefData (machP, cdP, rfcdP, refDepth) 
  machineType    * machP     ;   coverDataType  * cdP  ;
  refinedCDtype  * rfcdP     ;   int              refDepth  ;   
{
   int i ;

   (rfcdP->refDepth) = refDepth ;
   (rfcdP->ulongBitNum) = (machP->ulongBitNum) ;   
   if ((refDepth >= 0) && (refDepth+1 < (machP->ulongBitNum))) 
   {  (rfcdP->numSets) = 1 ;
      for (i = 0; i < refDepth-1; i++) (rfcdP->numSets) *= 2 ;    }
   else
   {  printf ("\"InitRefData\" got a bad value " ) ;
      printf ("(%d) for \"refDepth.\"\n", refDepth) ;
      fflush(stdout) ; exit(EXIT_FAILURE) ;   }

   (rfcdP->denom) =  2 * (rfcdP->numSets) ;
   (rfcdP->numCellSizes) = (rfcdP->numSets) * (cdP->numCellSizes) ;
   (rfcdP->embedDim) = (cdP->embedDim) ;

   return ;
}
/* ######################################################### */
/*                       GetRefStatRAM                       */
/* ######################################################### */
/*
    THIS PROCEDURE ALLOCATES THE ARRAY STORAGE NEEDED FOR
    REFINED DIMENSION ESTIMATES.  THE LENGTH,
    rfcdP->numCellSizes, OF THE ARRAYS DEPENDS ON THE "DEPTH"
    OF THE REFINEMENT.  IN GENERAL, rfcdP->numCellSizes =
    (2^^(refDepth-1)) * (cdP->numCellSizes).  ON A 32-BIT
    MACHINE, (cdP->numCellSizes) IS 33.  A DEPTH OF 5 YIELDS
    rfcdP->numCellSizes = 16 * 33 = 528.  SO WE'D WORK WITH A
    RANGE OF 528 CELL SIZES IN THIS CASE.
*/
void  GetRefStatRAM (rp)
        refinedCDtype * rp ;
{  
   char s[14];
   size_t   ulS = sizeof(uLong),        dS    = sizeof(double) ;
   size_t   nCS = (size_t) (rp->numCellSizes) ;

   strcpy(s, "GetRefStatRAM") ;
   
   (rp->scaleIndex)     = (uLong *)  gmalloc(s, "scaleIndex",      nCS * ulS);
   (rp->logCellSize)    = (double *) gmalloc(s, "logCellSize",     nCS * dS ); 
   (rp->cellCount)      = (uLong *)  gmalloc(s, "cellCount",       nCS * ulS);
   (rp->negLogCellCount)= (double *) gmalloc(s, "negLogCellCount", nCS * dS );
   (rp->information)    = (double *) gmalloc(s, "information",     nCS * dS );
   (rp->logSumSqrFreq)  = (double *) gmalloc(s, "logSumSqrFreq",   nCS * dS );

   return ;
}
/* ######################################################### */
/*                       XferSweepRes                        */
/* ######################################################### */
/*
    PLACE THE RESULTS FROM A STANDARD SWEEP INTO ONE OF THE
    DATA STRUCTURES FOR THE REFINED COVER DATA.  THIS PROCEDURE
    HAS TO BE CALLED SEVERAL TIMES IN ORDER TO INTERLEAVE
    RESULTS FROM SEVERAL SWEEPS INTO THE ARRAYS IN rfcdP.
*/
void  XferSweepRes (cdP, rfcdP, offset)
  coverDataType  * cdP   ;
  refinedCDtype  * rfcdP ;
  uLong            offset ;
{
   double ln2 = (cdP->log2), temp ;
   uLong cdIdx, rfcdIdx, nS = rfcdP->numSets, si = 2*nS-offset,
         denom = (rfcdP->denom) ;

   for (cdIdx = 0; cdIdx < cdP->numCellSizes; cdIdx++)
   {
     rfcdIdx = nS * cdIdx + offset ;
     (rfcdP->scaleIndex) [rfcdIdx]     = si ;
     temp                              = log(denom/((float)si) ) ;
     (rfcdP->logCellSize)[rfcdIdx]     = (temp/ln2) + cdIdx ;
     (rfcdP->cellCount)[rfcdIdx]       = (cdP->cellCount)[cdIdx];
     (rfcdP->negLogCellCount)[rfcdIdx] = (cdP->negLogCellCount)[cdIdx];
     (rfcdP->information)[rfcdIdx]     = (cdP->information)[cdIdx] ;
     (rfcdP->logSumSqrFreq)[rfcdIdx]   = (cdP->logSumSqrFreq)[cdIdx] ;
   }
   return ;
}
/* ######################################################### */
/*                        DoRescales                         */
/* ######################################################### */
/*
    THIS FUNCTION REPEATEDLY RESCALES THE DATA AND RUNS A LOAD
    AND SWEEP.  THE EFFECT IS TO GET COVERING STATISTICS OVER A
    MORE FINELY-DIVIDED RANGE OF CELL SIZES.  FOR EXAMPLE,
    INSTEAD OF HAVING CELL SIZES OF 1, 2, 4, 8, ETC., ONE CAN
    HAVE CELL SIZES OF 1, 1.5, 2, 3, 4, 6, 8.  FUNCTION
    DoCommand CALLS FUNCTION BuildDataRep, WHICH IN TURN CALLS
    GetNormztnInfo IN FILE datarep.c.  GetNormztnInfo COMPUTES
    MAX'S AND MIN'S IN THE DATA SET, AND ASSIGNS THE SCALING
    FACTOR WHICH IS TO BE USED FOR A STANDARD RUN OF FD3.  THE
    FUNCTION BELOW, DoRescales, ASSUMES THAT GetNormztnInfo HAS
    ALREADY RUN.  FOR A RANGE OF SCALE SHIFTS, IT MODIFIES THE
    SCALING FACTOR POINTED TO BY nrmInfP APPROPRIATELY AND
    RE-INITIALIZES DATA STRUCTURES.  IT THEN CALLS LoadRAM,
    WHICH USES THE MODIFIED SCALING FACTOR AS IT LOADS THE DATA
    FROM THE FILE.  THE RESULTING LOADED IMAGE IS THEN SORTED
    AND SWEPT, AND THE RESULTING COVERING STATISTICS ARE PLACED
    IN THE STRUCTURE *rfcdP, WHERE SOME FUNCTION FOR ESTIMATING
    DIMENSION CAN USE THEM.

    WE ACHIEVE THE DESIRED EFFECT, BUT GO SLOWLY BECAUSE DATA
    IS READ FROM DISK OVER AND OVER.  ONE ALTERNATIVE WOULD BE
    TO RE-SCALE THE DATA IN RAM.  THE TROUBLE WITH THAT METHOD
    IS THAT ROUND-OFF ERRORS COULD SPOIL THE DATA AFTER SEVERAL
    RE-SCALINGS, UNLESS WE SOMEHOW SAVE A COPY OF THE ORIGINAL
    DATA.  ANOTHER POSSIBILITY IS TO LOAD THE DATA AS DOUBLES
    INSTEAD OF AS ULONGS.  THE SWEEP AND SORT ROUTINES COULD
    APPLY SCALING INCREMENTALLY IN ORDER TO PERFORM THE BIT
    OPERATIONS THEY NEED TO DO.  OBVIOUSLY THERE WOULD BE A
    PERFORMANCE PENALTY FOR THIS, ESPECIALLY IN THE RADIX SORT,
    SINCE IT PASSES OVER THE DATA SEVERAL TIMES.

    IF THE EMBEDDING DIMENSION IS ONE, THERE IS REALLY NO NEED
    TO RE-SORT.  UNFORTUNATELY, THIS DOES NOT GENERALIZE TO
    HIGHER EMBEDDING DIMENSIONS.
*/
void DoRescales (cmdP, frP, rdP, cdP, niP, rfcdP, maxD)
       commandType        * cmdP      ;
       fileRepType        * frP       ;
       ramDataRepType     * rdP       ;
       coverDataType      * cdP       ;
       normInfoType       * niP       ;
       refinedCDtype      * rfcdP     ;
       double               maxD      ;
{
   double sF = niP->scalingFactor ;
   uLong  nS = rfcdP->numSets, i ;

   for (i = 1; i < nS; i++) 
   {
      (niP->scalingFactor) = sF * (2*nS-i)/(2*nS) ;
      ClearQ     ( &( (rdP->Q)[0] ) ) ;
      ClearQ     ( &( (rdP->Q)[1] ) ) ;
      LoadRAM    (cmdP,  frP,  rdP, niP, maxD) ;
      SortRAM    (cmdP,  rdP) ;
      if (!(cmdP->terse))
         { printf("DoRescales: Getting the covering statistics ... ") ;
           fflush(stdout); }
      StdDataSweep  (rdP, cdP) ;
      if (!(cmdP->terse)) { printf("Done.\n") ; fflush(stdout); }
      XferSweepRes (cdP, rfcdP, i ) ;
   }
   return ;
}
/* ######################################################### */
/*                     ReportRefCovD                         */
/* ######################################################### */
/*
      PRINT OUT THE TABLE CONTAINING REFINED COVERING
      STATISTICS.
*/
void ReportRefCovD (rfcdP)
       refinedCDtype     * rfcdP  ;
{
   CSTtitles         ()        ;
   RCSTbody          (rfcdP)   ;
   RfTwoPtEstTitles  ()        ;
   RfTwoPtEstBody    (rfcdP)   ;
   return ;
}
/* ######################################################### */
/*                         RCSTbody                          */
/* ######################################################### */
/*
    FOR THE refined (-f) OPTION, PRINT OUT THE FIRST TABLE OF
    COVERING STATISTICS.
*/
void RCSTbody (rfcdP)
       refinedCDtype     * rfcdP  ;
{
  uLong j, satPt, finePt ;
  
  satPt = (rfcdP->saturationPoint) ;
  finePt = (rfcdP->fineMeshPoint) ;
  for(j = 0; j < (rfcdP->numCellSizes); j++)
  {
     if ( (satPt < finePt) && (j == satPt ) ) MakeStarBar ();
     printf( "%5lu", j);
     printf("%12.1f",     (rfcdP->logCellSize)       [j]   );
     printf( "%11lu",     (rfcdP->cellCount)         [j]   );
     printf( "%16.5f",   -((rfcdP->negLogCellCount ) [j] ) );
     printf( "%11.5f",   -((rfcdP->information     ) [j] ) );
     printf( "%21.5f\n", -((rfcdP->logSumSqrFreq   ) [j] ) );
     if ( (satPt < finePt) && (j == finePt ) ) MakeStarBar ();
  }
  return ;
}
/* ######################################################### */
/*                      RfTwoPtEstTitles                     */
/* ######################################################### */
/*
     FOR THE refined (-f) OPTION, MAKE THE COLUMN TITLES FOR
     TABLE TWO -- THE TABLE OF TWO-POINT ESTIMATES.
*/
void RfTwoPtEstTitles () 
{
  printf ("\n\n\n");
  printf ("Two-Point Estimates of FD's ");
  printf ("\n\n");
  printf("j = index\n") ;
  printf("e = e[j]      = jth cell size (side length, in fd3's scale)\n") ;
  printf("delta(e)      = log(e[j+1])    -  log(e[j])\n") ;
  printf("delta(log(N)) = log(N(e[j]))   -  log(N(e[j+1]))\n") ;
  printf("delta(I)      = I(e[j])        -  I(e[j+1])\n") ;
  printf("delta(logSSF) = logSSF(e[j+1]) -  logSSF(e[j])\n") ;
  printf ("\n\n");
  printf("index      log(e)              ");
  printf("delta(log(N))   delta(I)        delta(logSSF)");
  printf("\n");
  printf("                               ");
  printf("-------------   --------        -------------") ;
  printf("\n");
  printf("                               ");
  printf("   delta(e)     delta(e)           delta(e)  ") ;
  printf("\n\n");
  return ;
}
/* ######################################################### */
/*                     RfTwoPtEstBody                        */
/* ######################################################### */
/*
   FOR THE refined (-f) OPTION, PRINT OUT THE TABLE OF
   TWO-POINT ESTIMATES.
*/
void RfTwoPtEstBody (r)
       refinedCDtype     * r  ;
{
  uLong j, satPt, finePt ;
  double dTemp, lDiff ;

  satPt = (r->saturationPoint) ;
  finePt = (r->fineMeshPoint) ;
  for(j = 0; j < (r->numCellSizes)-1; j++)
  {
     if ( (satPt < finePt) && (j == satPt ) ) MakeStarBar ();

     lDiff = (r->logCellSize)[j+1] -((r->logCellSize)[j]) ;

     printf( "%5lu", j);
     printf("%12.2f", (r->logCellSize)[j]);

     dTemp = ((r->negLogCellCount)[j+1]-(r->negLogCellCount)[j])/lDiff ;
     printf("%27.5f", dTemp);

     dTemp = ((r->information)[j+1]-(r->information)[j])/lDiff ;
     printf("%11.5f", dTemp);

     dTemp = ((r->logSumSqrFreq)[j+1]-(r->logSumSqrFreq)[j])/lDiff ;
     printf ("%21.5f", dTemp);

     printf ("\n") ;
     if ( (satPt < finePt) && (j+1 == finePt ) ) MakeStarBar ();
  }
  return ;
}
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */     
/*                      end coversf.c                        */
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */
