
/* BEGIN COPYRIGHT NOTICE

Copyright (c) 1992 by John Sarraille and Peter DiFalco
(john@ishi.csustan.edu)

Copyright (c) 1998 by John Sarraille
(john@ishi.csustan.edu)

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that copyright notice and this permission
notice appear in supporting documentation.

The algorithm used in this program was inspired by the paper
entitled "A Fast Algorithm To Determine Fractal Dimensions By
Box Counting", which was written by Liebovitch and Toth, and
which appeared in the journal "Physics Letters A", volume 141,
pp 386-390, (1989).

This program is not warranted: use at your own risk.

END COPYRIGHT NOTICE */


/*

   FD3 PROGRAM FILE

Filename:  docom.c

OUTLINE OF CALLS:

DoCommand +
  DCmsg1 +
    PrRepHead +
  GetFileRep +
    Gfopen
    GetDataLines +
      SkipSameLineWhite
      GetUlong
    SkipSameLineWhite
    GetEmbedDim
    fclose
  mOptDimErr
  BuildDataRep +
    GetNormztnInfo
    GetRAMsize
    GetRAM
    InitRAM
    LoadRAM
    SortRAM
  GetCoverData +
    InitCovData 
    GetStatRAM 
    InitCellSizes
    PrintInitCovData
    DBSweepData
    SweepData 
    FindRange 
  DoRefinedEsts +
    InitRefData
    GetRefStatRAM
    XferSweepRes
    DoRescales
    FindRefRange
    EstRefDims +
      InitRefEstD
      MakeRefFits
    REreport +
      PrRepHead +
      ReportRefCovD
      ReportDimEsts
  DoCoarseEsts +
    EstimateDims +
      InitEstData 
      MakeFits
    DCreport +
      PrRepHead +
      ReportCovData
      ReportDimEsts
  DCmsg2 +

*/

#include "fd.h"

/*
     THE FUNCTIONS IN THIS FILE:
*/
void DoCommand       () ;
void DCmsg1          () ;
void PrRepHead       () ;
void GetFileRep      () ;
void GetDataLines    () ;
void BuildDataRep    () ;
void GetCoverData    () ;
void DoRefinedEsts   () ;
void EstRefDims      () ;
void REreport        () ;
void DoCoarseEsts    () ;
void EstimateDims    () ;
void DCreport        () ;
void DCmsg2          () ;

/*
     THE FUNCTIONS NOT IN THIS FILE, BUT CALLED FROM HERE
*/
extern void    mOptDimErr        () ;    /*  in optm.c      */
extern void    Gfopen            () ;    /*  in file.c      */
extern int     SkipSameLineWhite () ;    /*  in file.c      */
extern int     GetUlong          () ;    /*  in file.c      */
extern void    GetEmbedDim       () ;    /*  in filerep.c   */
extern void    PrintFileRep      () ;    /*  in filerep.c   */
extern void    GetNormztnInfo    () ;    /*  in datarep.c   */
extern void    GetRAMsize        () ;    /*  in datarep.c   */
extern void    GetRAM            () ;    /*  in datarep.c   */
extern void    InitRAM           () ;    /*  in datarep.c   */
extern void    LoadRAM           () ;    /*  in datarep.c   */
extern void    SortRAM           () ;    /*  in datarep.c   */
extern void    InitCovData       () ;    /*  in covers.c    */
extern void    GetStatRAM        () ;    /*  in covers.c    */
extern void    InitCellSizes     () ;    /*  in covers.c    */
extern void    PrintInitCovData  () ;    /*  in covers.c    */
extern void    SweepData         () ;    /*  in covers.c    */
extern void    DBSweepData       () ;    /*  in coversdb.c  */
extern void    FindRange         () ;    /*  in covers.c    */
extern void    FindRefRange      () ;    /*  in coversf.c   */
extern void    InitRefData       () ;    /*  in coversf.c   */
extern void    GetRefStatRAM     () ;    /*  in coversf.c   */
extern void    XferSweepRes      () ;    /*  in coversf.c   */
extern void    DoRescales        () ;    /*  in coversf.c   */
extern void    InitRefEstD       () ;    /*  in estim.c     */
extern void    MakeRefFits       () ;    /*  in estim.c     */
extern void    ReportRefCovD     () ;    /*  in coversf.c   */
extern void    ReportCovData     () ;    /*  in covers.c    */
extern void    InitEstData       () ;    /*  in estim.c     */
extern void    MakeFits          () ;    /*  in estim.c     */
extern void    ReportDimEsts     () ;    /*  in estim.c     */

/* ######################################################### */
/*                       DoCommand                           */
/* ######################################################### */
void DoCommand (cmdP, machP)
       commandType * cmdP ;   machineType * machP ;
  {
    fileRepType       fRep ;  ramDataRepType    ramD ;
    coverDataType     covD ;  dimEstimDataType  deD ;
    normInfoType      ni ;

    DCmsg1         (cmdP)  ; /* PRINTS SOME TEXT */
    GetFileRep     (cmdP, machP, &fRep) ;
    if ( (cmdP->mincover) && ((fRep.embedDim) != (uLong) 1) )
       mOptDimErr(&fRep) ;
    
    BuildDataRep   (cmdP, machP, &fRep, &ramD, &ni) ;
    GetCoverData   (cmdP, &ramD, &covD, &ni) ;
    if    (cmdP->fine)
             /*
                  DO REFINED ESTIMATION SO AS TO GET 4 DIFFERENT
                  SCALINGS OF THE DATA.  (refDepth==3)
             */
          DoRefinedEsts (cmdP, machP, &fRep, &ramD, &covD, &deD, &ni, 3) ;
    else  DoCoarseEsts (&fRep, &covD, &deD) ;
    DCmsg2 (cmdP) ;
    return ;
  }
/* ######################################################### */
/*                        DCmsg1                             */
/* ######################################################### */
void DCmsg1 (cmdPtr) 
       commandType * cmdPtr ;
{
   
   if ( (cmdPtr->debugging) || (cmdPtr->checking) )
       { printf ("THE OUTPUT OF DoCommand IS: \n\n");
         fflush(stdout);  
       }

   PrRepHead (cmdPtr->infileName) ; /* PRINT THE REPORT HEADING. */
   return ;
}
/* ######################################################### */
/*                       GetFileRep                          */
/* ######################################################### */
/* 
     HERE WE FILL UP A RECORD WHICH REPRESENTS THE INPUT FILE.
     WE TRANSFER THE FILE NAME TO THE RECORD AND OPEN THE
     FILE.  WE THEN RECORD HOW MANY LINES OF DATA THE FILE HAS,
     AND HOW MANY COORDINATES ARE ON A LINE.

     DEVELOPER NOTES:
         DON'T CHANGE "rb+" FILE MODE.  WE WANT THE fseeks OF
         THE CHARACTER-SKIPPING UTILITY FUNCTIONS TO BE
         GUARANTEED TO WORK CORRECTLY.
*/
void GetFileRep (commandPtr, machinePtr, fileRepPtr) 
       commandType * commandPtr ;
       machineType *  machinePtr ;
       fileRepType * fileRepPtr ;
{
  strcpy (fileRepPtr->fileName, commandPtr->infileName) ;

  if ( (commandPtr->debugging) )
  {  printf("\"GetFileRep\" is about to try to open file: ") ;
     printf("%s.\n\n", fileRepPtr->fileName) ;  fflush(stdout);  }
  Gfopen ("GetFileRep", &(fileRepPtr->fileVarPtr),
                        fileRepPtr->fileName, "rb+");

  GetDataLines (commandPtr, machinePtr, fileRepPtr) ;

  SkipSameLineWhite (commandPtr, fileRepPtr->fileVarPtr) ; 
  GetEmbedDim(commandPtr, fileRepPtr) ;

  fclose (fileRepPtr->fileVarPtr) ;
  if ( (commandPtr->debugging) || (commandPtr->checking) )
              PrintFileRep (commandPtr, fileRepPtr) ;
  return ;
}
/* ######################################################### */
/*                      GetDataLines                         */
/* ######################################################### */
/*
    GET THE NUMBER OF LINES OF DATA IN THE FILE -- THIS IS
    SUPPOSED TO BE THE FIRST NUMBER IN THE FILE.
*/
void GetDataLines (commandPtr, machinePtr, fileRepPtr) 
       commandType * commandPtr ;
       machineType *  machinePtr ;
       fileRepType * fileRepPtr ;
{ int DLisOK ;

  if ( (commandPtr->debugging) )
  {  printf("\"GetDataLines\" is about to read \"dataLines\" ") ;
     printf("from file: %s\n\n", fileRepPtr->fileName) ;
     fflush(stdout);  }
  SkipSameLineWhite (commandPtr, fileRepPtr->fileVarPtr) ;
  DLisOK = GetUlong( commandPtr, fileRepPtr->fileVarPtr,
                     machinePtr->maxUlong,
                     &(fileRepPtr->dataLines) ) ;
  if ( !DLisOK || ( (fileRepPtr->dataLines) < 1)  )
  { printf("\"GetDataLines\" got a bad value for \"dataLines\"\n") ;
    printf("Check the first line of the input file: ") ;
    printf("%s.\n\n", fileRepPtr->fileName) ;
    fflush(stdout);  exit(EXIT_FAILURE) ;
  }
  return ;
}
/* ######################################################### */
/*                       BuildDataRep                        */
/* ######################################################### */
void BuildDataRep (cmdP, machP, fRepP, rDptr, nrmInfP)
        commandType    * cmdP;       machineType    * machP;
        fileRepType    * fRepP;      ramDataRepType * rDptr ;
        normInfoType   * nrmInfP ;

  {
    GetNormztnInfo (cmdP,  machP,  fRepP, nrmInfP);
    GetRAMsize     (machP, fRepP,  rDptr);
    GetRAM         (cmdP,  rDptr);
    InitRAM        (rDptr);
    LoadRAM        (cmdP,  fRepP,  rDptr, nrmInfP, machP->maxDouble) ;
    SortRAM        (cmdP,  rDptr);

    return ;
  }
/* ######################################################### */
/*                      GetCoverData                         */
/* ######################################################### */
/*
    THE JOB OF THIS FUNCTION IS TO FIRST INITIALIZE THE DATA
    STRUCTURE THAT HOLDS CELL COUNTS AND OTHER COVERING
    STATISTICS, AND THEN TO CALL THE FUNCTIONS THAT COMPUTE
    THESE QUANTITIES.
*/

void GetCoverData (cmdPtr, ramDataPtr, covDptr, nrmInfPtr)
      commandType    * cmdPtr ;
      ramDataRepType * ramDataPtr ;
      coverDataType  * covDptr  ;
      normInfoType   * nrmInfPtr ;
{
   InitCovData (cmdPtr, ramDataPtr, covDptr, nrmInfPtr) ;
   GetStatRAM (covDptr);
   InitCellSizes (cmdPtr, covDptr, nrmInfPtr);
   if ((cmdPtr->debugging) || (cmdPtr->checking))
      PrintInitCovData(covDptr) ;
   if (!(cmdPtr->terse))
      { printf("GetCoverData: Getting the covering statistics ... ") ;
        fflush(stdout); }
   SweepData (cmdPtr, ramDataPtr, covDptr) ;
   if (!(cmdPtr->terse)) { printf("Done.\n") ; fflush(stdout); }
   FindRange (covDptr, cmdPtr) ;
   return ;
}
/* ######################################################### */
/*                      DoRefinedEsts                        */
/* ######################################################### */
/*
     THIS PROCEDURE DOES DIMENSION ESTIMATES USING MORE CELL
     SIZES THAN THE ORDINARY FD3 RUN.  AN EXPONENT s=refDepth
     IS INPUT AS A PARAMETER.  refDepth=s DETERMINES A
     DENOMINATOR OF THE FORM 2^^s, AND THE DATA IS SCALED WITH
     2^^(s-1) SCALING FACTORS.  THE FACTORS ARE
     (2^^(s-1)+i)*SF/2^^s, FOR i = 1, 2, ..., 2^^(s-1), WHERE
     SF IS THE BASIC SCALING FACTOR CHOSEN BY THE FD3 SOFTWARE
     -- maxUlong/maxDiff. FOR EXAMPLE, WHEN s=5, FACTORS OF
     17*SF/32, 18*SF/32, ..., 32*SF/32 ARE USED.  
*/
void DoRefinedEsts (cmdP, machP, frP, rdP, cdP, deP, niP, refDepth) 
       commandType     * cmdP  ;    machineType        * machP     ;
       fileRepType     * frP   ;    ramDataRepType     * rdP       ;
       coverDataType   * cdP   ;    dimEstimDataType   * deP       ;
       normInfoType    * niP   ;    int                  refDepth  ;
{
    refinedCDtype  rfcd ;

    InitRefData    (machP, cdP, &rfcd, refDepth) ;  
    GetRefStatRAM  (&rfcd) ;
    XferSweepRes   (cdP, &rfcd, (uLong) 0) ;
    DoRescales     (cmdP, frP, rdP, cdP, niP, &rfcd, machP->maxDouble) ;
    FindRefRange    (&rfcd, cmdP) ;
    EstRefDims     (deP, &rfcd) ;
    REreport       (frP, deP, &rfcd) ; 
    return ;
}
/* ######################################################### */
/*                       EstRefDims                          */
/* ######################################################### */
void EstRefDims    (deP, rfcdP)
       dimEstimDataType * deP ;   refinedCDtype * rfcdP ;
{
   InitRefEstD (deP, rfcdP) ;
   MakeRefFits    (deP, rfcdP) ;

   return ;
}
/* ######################################################### */
/*                        REreport                           */
/* ######################################################### */
void REreport (frP, deP, rfcdP) 
         fileRepType       * frP    ;
         refinedCDtype     * rfcdP  ;
         dimEstimDataType  * deP    ;
{
   PrRepHead (frP->fileName) ;
   ReportRefCovD  (rfcdP)  ; 
   ReportDimEsts  (deP) ;

   return ;
}
/* ######################################################### */
/*                        PrRepHead                          */
/* ######################################################### */
/*
    THIS FUNCTION PRINTS A HEADING FOR THE REPORT THAT IS
    OUTPUT BY THE PROGRAM.
*/
void PrRepHead (fileName) 
       char * fileName ;
{
   int i ;
   
   printf("\n\n") ;
   for (i=0; i < 58; i++)  printf("*") ;
   printf ("\n") ;
   printf("  FRACTAL DIMENSION REPORT ");
   printf("-- FD3 software (version 0.4)");
   printf("\n") ;
   for (i=0; i < 58; i++)  printf("*") ;
   printf ("\n\n") ;
   printf ("Reporting on file named: %s.", fileName) ;
   printf ("\n\n") ;

   return ;
}   
/* ######################################################### */
/*                      DoCoarseEsts                         */
/* ######################################################### */
/*
    THIS PROCEDURE MAKES THE PROPER CALLS TO GET THE STANDARD
    FD3 ESTIMATES.
*/
void  DoCoarseEsts (frP, cdP, deP) 
   fileRepType       * frP   ;
   coverDataType     * cdP   ;
   dimEstimDataType  * deP   ;
{
   EstimateDims   (cdP, deP) ;
   DCreport       (frP, cdP, deP); 

   return ;
}
/* ######################################################### */
/*                      EstimateDims                         */
/* ######################################################### */
void EstimateDims (covDataPtr, dimEstDataPtr)
       coverDataType     * covDataPtr ;
       dimEstimDataType  * dimEstDataPtr ;
{
   InitEstData (covDataPtr, dimEstDataPtr) ;
   MakeFits (covDataPtr, dimEstDataPtr) ;

   return ;
}
/* ######################################################### */
/*                       DCreport                            */
/* ######################################################### */
void  DCreport (fileRepP, covDataP, dimEstDataP)
         fileRepType       * fileRepP ;
         coverDataType     * covDataP    ;
         dimEstimDataType  * dimEstDataP ;
{
   PrRepHead (fileRepP->fileName) ;
   ReportCovData  (covDataP)  ;
   ReportDimEsts  (dimEstDataP) ;

   return ;
}
/* ######################################################### */
/*                        DCmsg2                             */
/* ######################################################### */
void DCmsg2 (cmdPtr) 
       commandType * cmdPtr ;
{
   if ( (cmdPtr->debugging) || (cmdPtr->checking) )
       {  printf ("DoCommand ran to a successful completion.\n\n") ;
          fflush(stdout);
       }
   return ;
}
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */     
/*                       end docom.c                         */
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */
