
/* BEGIN COPYRIGHT NOTICE

Copyright (c) 1992 by John Sarraille and Peter DiFalco
(john@ishi.csustan.edu)

Copyright (c) 1998 by John Sarraille
(john@ishi.csustan.edu)

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that copyright notice and this permission
notice appear in supporting documentation.

The algorithm used in this program was inspired by the paper
entitled "A Fast Algorithm To Determine Fractal Dimensions By
Box Counting", which was written by Liebovitch and Toth, and
which appeared in the journal "Physics Letters A", volume 141,
pp 386-390, (1989).

This program is not warranted: use at your own risk.

END COPYRIGHT NOTICE */


/*

   FD3 PROGRAM FILE

Filename:  filerep.c

OUTLINE OF CALLS:

PrintFileRep +
   PrintFileLines
GetEmbedDim +
   CkDBug +
   SkipSameLineWhite
   CkEOF +
   SkipEOLNchars
   CkExtrI +
   CkExtrEL +
   CountStringsOnLine
   CkWhiteL +

*/

#include "fd.h"

/*
     THE FUNCTIONS IN THIS FILE:
*/
void  PrintFileRep () ;
void  GetEmbedDim  () ;
void  CkDBug       () ;
void  CkEOF        () ;
void  CkExtrI      () ;
void  CkExtrEL     () ;
void  CkWhiteL     () ;

/*
     THE FUNCTIONS NOT IN THIS FILE, BUT CALLED FROM HERE:
*/
extern void PrintFileLines      () ;     /* in file.c   */
extern int  SkipSameLineWhite   () ;     /* in file.c   */
extern int  SkipEOLNchars       () ;     /* in file.c   */
extern int  CountStringsOnLine  () ;     /* in file.c   */

/* ######################################################### */
/*                   PrintFileRep                            */
/* ######################################################### */
void PrintFileRep (commandPtr, fileRepPtr)
       commandType * commandPtr ;
       fileRepType * fileRepPtr ;
{
  uLong startLineNum = 1 , stopLineNum = 5;
  printf("The FILE REPRESENTATION is: \n\n") ;
  printf ("fileName    = %s\n", fileRepPtr->fileName) ;
  printf ("embedDim    = %lu\n", fileRepPtr->embedDim) ;
  printf ("dataLines   = %lu\n\n", fileRepPtr->dataLines) ;
  printf ("The first 5 lines of the file are ==>\n\n") ;
  PrintFileLines (commandPtr, fileRepPtr->fileName,
                     startLineNum, stopLineNum) ;
  printf ("\n") ;  fflush(stdout);    
  return ;
}
/* ######################################################### */
/*                     GetEmbedDim                           */
/* ######################################################### */
/*
   THIS FUNCTION USES CALLS TO VARIOUS "SKIPPERS" -- FILE
   UTILITY COMMANDS -- TO FIGURE OUT THE NUMBER OF STRINGS ON
   THE SECOND LINE OF AN FD3 INPUT FILE.  THIS NUMBER OF
   STRINGS WILL BE THE EMBEDDING DIMENSION IF THE USER HAS
   CORRECTLY STRUCTURED THE INPUT FILE.  THE UTILITY FUNCTIONS
   USE fseek TO "BACK UP ONE CHARACTER" AFTER FINDING THE END
   OF A STRING OR CONTIGUOUS SECTION OF WHITE SPACE, OR THE
   LIKE.  UNFORTUNATELY fseek IS NOT GUARANTEED TO WORK ON A
   DOS FILE OPENED IN ASCII MODE, SO WE MUST MAKE SURE THAT THE
   FILE IS OPENED IN BINARY MODE IF THE PLATFORM IS A DOS
   MACHINE.  EVIDENTLY THE CODES THAT DO THIS (e.g. "rb") DO
   NOT CAUSE A PROBLEM FOR UNIX MACHINES.  SO FAR THIS
   HYPOTHESIS HAS ONLY BEEN TESTED ON A SUN SPARCSTATION.  IF
   WE HAVE PROBLEMS WITH THIS APPROACH LATER, WE COULD USE THE
   ungetc FUNCTION INSTEAD OF A SEEK.  THERE IS ALSO THE
   POSSIBILITY OF USING ifdef DIRECTIVES TO OPEN THE FILE
   DIFFERENTLY DEPENDING ON WHETHER THE PLATFORM IS A DOS
   MACHINE.  I DON'T HAVE EXPERIENCE DOING THAT, AND I'M
   RELUCTANT TO LEARN FOR THE TIME BEING. :-)

   A PRECONDTION HERE IS THAT THE LAST CHARACTER IN THE
   VARIABLE dataLines HAS JUST BEEN READ SUCCESSFULLY, AND THAT
   the dataLines VALUE IS THE ONLY THING ON THE FIRST LINE OF
   THE FILE OTHER THAN WHITE SPACE.  IF SO, THE CALL TO SKIP
   OVER THE NEXT BLOCK OF WHITE SPACE ON THE CURRENT LINE
   SHOULD MOVE THE FILE POSITION TO THE END OF THE LINE.

   AFTER "TAKING CARE OF EXTRANEOUS INPUT ERROR" WE ASSUME THAT
   WE ARE AT THE END OF LINE ONE OF THE INPUT FILE.  WE THEN
   SKIP END-OF-LINE CHARACTERS, TO GET TO THE BEGINNING OF THE
   ACTUAL DATA IN THE FILE.  WE ASSUME THAT THERE IS SUPPOSED
   TO BE AT LEAST ONE LINE OF DATA, AND THAT IT IS SUPPOSED TO
   START ON THE SECOND LINE.
*/   
void GetEmbedDim (commandPtr, fileRepPtr)
       commandType * commandPtr ;
       fileRepType * fileRepPtr ;
{
  int   curCh, lines, count   ;

  CkDBug (commandPtr, fileRepPtr) ;

  curCh = SkipSameLineWhite (commandPtr, fileRepPtr->fileVarPtr);
  CkEOF (curCh, fileRepPtr) ;
  CkExtrI (curCh, fileRepPtr) ;

  lines = SkipEOLNchars (commandPtr, fileRepPtr->fileVarPtr) ;
  CkExtrEL (lines, fileRepPtr) ;

  count = CountStringsOnLine (commandPtr, fileRepPtr->fileVarPtr) ;
  CkWhiteL (count, fileRepPtr) ;

  (fileRepPtr->embedDim) = count ;
  return ;
}
/* ######################################################### */
/*                         CkDBug                            */
/* ######################################################### */
/*
      IF THE DEBUG FLAG IS ON, WARN THE USER ABOUT THE FILE
      INPUT THAT IS ABOUT TO OCCUR.
*/
void  CkDBug (commandPtr, fileRepPtr) 
       commandType * commandPtr ;
       fileRepType * fileRepPtr ;
{
  if ( (commandPtr->debugging) )
  {
     printf("\"GetEmbedDim\" is about to check the embedding ") ;
     printf("dimension in file: %s.\n\n", fileRepPtr->fileName) ;
     fflush(stdout);
  }
  return ;
}
/* ######################################################### */
/*                          CkEOF                            */
/* ######################################################### */
/*
          TAKE CARE OF PREMATURE EOF ERROR.
*/
void  CkEOF (curCh, fileRepPtr) 
         int           curCh ;
         fileRepType   * fileRepPtr ;
{
  if ( (curCh == EOF) )
  {
     printf( "\"CkEOF\" found premature end-of-file ") ;
     printf("on line #1 of file: %s.\n\n", fileRepPtr->fileName) ;
     printf("Please correct this, and any other format errors in the file.");
     fflush(stdout); exit(EXIT_FAILURE) ;
  }
  return ;
}
/* ######################################################### */
/*                        CkExtrI                            */
/* ######################################################### */
/*
          TAKE CARE OF EXTRANEOUS INPUT ERROR.
*/
void  CkExtrI (curCh, fileRepPtr)
         int           curCh ;
         fileRepType   * fileRepPtr ;
{
  if (    (((char) curCh) != '\n')
       && (((char) curCh) != '\r')
       && (((char) curCh) != '\f')  )
  {
     printf( "\"CkExtrI\" found extraneous character(s) ") ;
     printf("on line #1 of file: %s.\n", fileRepPtr->fileName) ;
     printf ("The \"offending\" character is ==> %c <==. \n\n", curCh) ;
     printf("Please correct this, and any other format errors in the file.");
     fflush(stdout); exit(EXIT_FAILURE) ;
  }
  return ;
}
/* ######################################################### */
/*                        CkExtrEL                           */
/* ######################################################### */
/*
     TAKE CARE OF EXTRANEOUS EMPTY LINES ERROR.
*/
void CkExtrEL (lines, fileRepPtr) 
         int           lines ;
         fileRepType   * fileRepPtr ;
{
  if (lines > 1) 
  {
    printf( "\"CkExtrEL\" FOUND EXTRANEOUS EMPTY LINES ") ;
    printf("AFTER FIRST LINE OF FILE: %s.\n\n", fileRepPtr->fileName) ;
    printf("Please correct this, and any other format errors in the file.");
    fflush(stdout); exit(EXIT_FAILURE) ;
  }
  return ;
}
/* ######################################################### */
/*                        CkWhiteL                           */
/* ######################################################### */
/*
       TAKE CARE OF LINE-WITH-WHITE-SPACE-ONLY ERROR.
*/
void CkWhiteL (count, fileRepPtr) 
         int           count ;
         fileRepType   * fileRepPtr ;
{
  if (count < 1)
  {
     printf( "\"CkWhiteL\" found only white space on ") ;
     printf("first data line of file: %s.\n\n", fileRepPtr->fileName) ;
     printf("Please correct this, and any other format errors in the file.");
     fflush(stdout); exit(EXIT_FAILURE) ;
  }

  return ;
}
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */     
/*                       end filerep.c                       */
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */
