
/* BEGIN COPYRIGHT NOTICE

Copyright (c) 1992 by John Sarraille and Peter DiFalco
(john@ishi.csustan.edu)

Copyright (c) 1998 by John Sarraille
(john@ishi.csustan.edu)

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that copyright notice and this permission
notice appear in supporting documentation.

The algorithm used in this program was inspired by the paper
entitled "A Fast Algorithm To Determine Fractal Dimensions By
Box Counting", which was written by Liebovitch and Toth, and
which appeared in the journal "Physics Letters A", volume 141,
pp 386-390, (1989).

This program is not warranted: use at your own risk.

END COPYRIGHT NOTICE */


/*

   FD3 PROGRAM FILE

Filename:   optp.c

OUTLINE OF CALLS:

pOptArgs +
  GetSatPt +
    GetULfromString
  GetFmeshPt +
    GetULfromString

*/

#include "fd.h"

/* 
    THE FUNCTIONS IN THIS FILE: 
*/
int    pOptArgs     () ;
int    GetSatPt     () ;
int    GetFmeshPt   () ;
/*   
      THE FUNCTIONS NOT IN THIS FILE,
      BUT CALLED FROM HERE:
*/
extern int      GetULfromString     () ;     /*  in convert.c    */

/* ######################################################### */
/*                         pOptArgs                          */
/* ######################################################### */
/*
     THIS FUNCTION TAKES CARE OF INPUTTING THE COMMAND-LINE
     ARGUMENTS, THAT ACCOMPANY THE -p OPTION.  THE ARGUMENTS
     ARE INTEGERS: A LOWER AND UPPER CELL SIZE NUMBER BETWEEN 0
     AND NUMCELLSIZES-1.  BOTH ARGUMENTS MUST BE GIVEN.
*/
int pOptArgs(argsLftPt, curArgPt, argv, cPt, mPt) 
       int * argsLftPt ;
       int * curArgPt ;
       char * argv[] ;
       commandType * cPt ;
       machineType * mPt ;
{
   int syntaxOK = FALSE ;
        /*
           IF THERE ARE TWO OR MORE STRINGS TO THE RIGHT OF "-p"
           (BESIDES THE FILENAME) ...
        */
  if ( ((* argsLftPt) >= 2))
        /*
           THEN, ASSUMING CORRECT SYNTAX, THE NEXT TWO STRINGS
           AFTER "-p" MUST BE ARGUMENTS FOR "-p", SO PARSE
           THEM, AND ACCORDINGLY UPDATE THE NUMBER OF ARGUMENTS
           LEFT TO PROCESS AND THE INDEX OF THE NEXT ARGUMENT
           TO PROCESS.
        */ 
  {
    syntaxOK = (GetSatPt(argv[(* curArgPt)], cPt, mPt))
                  && (GetFmeshPt(argv[(* curArgPt)+1], cPt, mPt))
                  && ((cPt->saturationPoint) < (cPt->fineMeshPoint)) ;
    (* curArgPt) =  (* curArgPt) + 2 ;
    (* argsLftPt) = (* argsLftPt) - 2 ;
  }
  return(syntaxOK) ;
}
/* ######################################################### */
/*                         GetSatPt                          */
/* ######################################################### */
/*
   GET THE INDEX OF THE CELL SIZE FOR THE SATURATION POINT.
*/
int GetSatPt(strIntPtr, cPt, mPt)
       char *         strIntPtr ;
       commandType *  cPt ;
       machineType *  mPt ;
{  
   int valueOK ;
   uLong value, ulongMax ;

   ulongMax = (mPt->maxUlong) ;
   valueOK = GetULfromString (strIntPtr, ulongMax, &value) ;
   if (valueOK)
      {
         (cPt->saturationPoint) = value ;
         return (TRUE) ;
      }
   else return (FALSE) ;
}
/* ######################################################### */
/*                        GetFmeshPt                         */
/* ######################################################### */
/*
   GET THE INDEX OF THE CELL SIZE FOR THE FINE MESH POINT.
*/
int GetFmeshPt(strIntPtr, cPt, mPt)
       char *         strIntPtr ;
       commandType *  cPt ;
       machineType *  mPt ;
{  
   int   valueOK ;
   uLong value, ulongMax ;

   ulongMax = (mPt->maxUlong) ;
   valueOK = GetULfromString (strIntPtr, ulongMax, &value) ;
   if (valueOK)
      {
         (cPt->fineMeshPoint) = value ;
         return (TRUE) ;
      }
   else return (FALSE) ;
}
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */     
/*                        end optp.c                         */
/* ######################################################### */
/* ######################################################### */
/* ######################################################### */
